@@ -103,7 +103,8 @@ convertScene <- function(x = scene3d(), width = NULL, height = NULL, reuse = NUL
 	flagnames <- c("is_lit", "is_smooth", "has_texture", "is_indexed",
 		       "depth_sort", "fixed_quads", "is_transparent",
 		       "is_lines", "sprites_3d", "sprite_3d",
-		       "is_subscene", "is_clipplanes")
+		       "is_subscene", "is_clipplanes",
+		       "fixed_size", "is_points", "is_twosided")
 
 	getFlags <- function(id) {
 
@@ -129,20 +130,27 @@ convertScene <- function(x = scene3d(), width = NULL, height = NULL, reuse = NUL
 		result["is_smooth"] <- mat$smooth && type %in% c("triangles", "quads", "surface", "planes",
 								 "spheres")
 
-		result["has_texture"] <- !is.null(mat$texture)
+		result["has_texture"] <- has_texture <- !is.null(mat$texture)
 
-		result["is_transparent"] <- is_transparent <- any(obj$colors[,"a"] < 1)
+		result["is_transparent"] <- is_transparent <- (has_texture && mat$isTransparent) || any(obj$colors[,"a"] < 1)
 
 		result["depth_sort"] <- depth_sort <- is_transparent && type %in% c("triangles", "quads", "surface",
 										    "spheres", "sprites", "text")
 
 		result["sprites_3d"] <- sprites_3d <- type == "sprites" && length(obj$ids)
 
-		result["is_indexed"] <- (depth_sort || type %in% c("quads", "surface", "text", "sprites")) && !sprites_3d
+		result["is_indexed"] <- (depth_sort ||
+					 type %in% c("quads", "surface", "text", "sprites") ||
+					 type %in% c("triangles") && length(intersect(c("points", "lines"), c(mat$front)))) && 
+			                !sprites_3d
 
 		result["fixed_quads"] <- type %in% c("text", "sprites") && !sprites_3d
 		result["is_lines"]    <- type %in% c("lines", "linestrip", "abclines")
-		result
+		result["is_points"]   <- type == "points" || "points" %in% c(mat$front, mat$back)
+		result["is_twosided"] <- type %in% c("quads", "surface", "triangles") && 
+			                   length(unique(c(mat$front, mat$back))) > 1
+	  result["fixed_size"]  <- type == "text" || isTRUE(obj$fixedSize)
+	  result
 	}
 
 	getSubsceneFlags <- function(id) {
@@ -205,7 +213,9 @@ convertScene <- function(x = scene3d(), width = NULL, height = NULL, reuse = NUL
 	                       subscene$par3d$bbox[3:4],
 	                       subscene$par3d$bbox[5:6])
 	    if (tempID > lastID)
-	      break;
+	      break
+	    else
+	      delFromSubscene3d(tempID)
 	  }
 
 	  # plot the clipping planes as they affect the bounding box
@@ -218,6 +228,7 @@ convertScene <- function(x = scene3d(), width = NULL, height = NULL, reuse = NUL
 	  if (any(inds <- is.na(verts[,1]) & is.na(verts[,2])))
 	    res <- c(res, do.call(axis3d, c(list(edge = "z", at = verts[inds, 3], labels = text[inds]), mat)))
 	  res <- c(res, do.call(box3d, mat))
+	  delFromSubscene3d(c(res, tempID))
 	  res
 	}
 
